﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LineManager : MonoBehaviour
{
    public float lineOffset;
    public int maxLineSize;
    public float ServicePeriod;
    public float ServiceValue;
    public SpriteRenderer holderRenderer;
    public int lineIndex;

    private Vector3[] waypoints;
    public List<Person> CurrentPersons;
    private IEnumerator ServiceCoroutine;
    
    void Awake()
    {
        CurrentPersons = new List<Person>();
    }

    void Start()
    {
        waypoints = new Vector3[maxLineSize];
        for (int i = 0; i < maxLineSize; i++)
        {
            waypoints[i] = new Vector3(transform.position.x - (i * lineOffset), transform.position.y, transform.position.z);
        }

        ServiceCoroutine = StartService();

        StartCoroutine(ServiceCoroutine);
    }

    // Update is called once per frame
    void Update()
    {

    }

    public bool isLineFull()
    {
        if(CurrentPersons.Count >= maxLineSize)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public Vector3 GetPlacePosition(int place)
    {
        return waypoints[place];
    }

    public int GetLineIndex()
    {
        return lineIndex;
    }

    public int OcuppyLinePlace(Person PersonRef)
    {
        CurrentPersons.Add(PersonRef);

        PersonRef.SetPlace(CurrentPersons.Count - 1);

        // if(CurrentPersons.Count == 1)
        // {
        //      StartCoroutine(ServiceCoroutine);
        // }

        return CurrentPersons.Count - 1;
    }

    public void DeocuppyLinePlace()
    {
        CurrentPersons.RemoveAt(0);

        Debug.Log("Line index: "+lineIndex);

        // if(CurrentPersons.Count <= 0)
        // {
        //      StopCoroutine(ServiceCoroutine);
        // }

        ShiftLine();
    }

    IEnumerator StartService()
	{
		while(true)
		{
            if(CurrentPersons.Count >= 1)
            {
                if(CurrentPersons[0].CanDoService() == true)
                {
                    CurrentPersons[0].DoService(ServiceValue);
                }
            }

			yield return new WaitForSeconds(ServicePeriod);
		}
	}

    public void RemovePersonAt(int index)
    {
        CurrentPersons.RemoveAt(index);
        ShiftLine(index);

    }

    void ShiftLine(int index)
    {
        for(int i = index; i < CurrentPersons.Count; i++)
        {
            Person person = CurrentPersons[i];
                
            int nextPlace = person.GetPlace() - 1;
            person.MoveTo(waypoints[nextPlace]);
            person.SetPlace(nextPlace);
        }
    }

    void ShiftLine()
	{
        foreach (Person person in CurrentPersons)
         {
             int nextPlace = person.GetPlace() - 1;
             person.MoveTo(waypoints[nextPlace]);
             person.SetPlace(nextPlace);
         }
	}

    public void ToggleHolder()
    {
        holderRenderer.enabled = !holderRenderer.enabled;
    }
}
