﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Person : MonoBehaviour {

	public AudioClip cashingClip;
	public AudioClip angryClip;
	public float Speed;
	public bool IsMoving;
	public bool hasProduct;
	private bool IsBeingHold;
	public float baseValue;
	public GameObject holdingPivot;
	private Vector3 TargetPosition;
	private Vector3[] PreviousWaypoints;
	private int NumWaypoints;
	public int CurrentWaypoint;
	public bool isSelectingLine;
	public bool isLeaving;
	public int CurrentPlace;
	public int CurrentLine;
	public float ServiceAmount;
	private ShopManager ShopManager;
	public LineManager CurrentLineManager;
	private GameManager gameManager;
	private Product productBeingHold;

	Animator animator;
	public float checkingDelay;

	//private SoundManager soundManager;

	void Start () {

		animator = GetComponent<Animator>();

		CurrentWaypoint = 0;
		isSelectingLine = true;
		isLeaving = false;

		ShopManager = GameObject.Find("Manager").GetComponent<ShopManager>();
		gameManager = GameObject.Find("GameManager").GetComponent<GameManager>();

		GameObject[] PreviousWaypointsObj = ShopManager.GetPreviousWaypointsObj();

		NumWaypoints = PreviousWaypointsObj.Length;

		PreviousWaypoints = new Vector3[NumWaypoints];

		for(int i = 0; i < NumWaypoints; i++)
		{
			PreviousWaypoints[i] = PreviousWaypointsObj[i].transform.position;
		}

		ReceiveProduct(ShopManager.ReturnRandomProduct());

		MoveNext();

		StartCoroutine(CheckServiceAmount());
	}
	
	void Update () {

		animator.SetBool("IsMoving", IsMoving);
		animator.SetBool("IsHolding", IsBeingHold);
		animator.SetBool("hasProduct", hasProduct);

		if(IsMoving)
		{
			float step = Speed * Time.deltaTime;
        	transform.position = Vector3.MoveTowards(transform.position, TargetPosition, step);

			if(Vector3.Distance(transform.position, TargetPosition) < 0.1f)
			{
				IsMoving = false;
				
				if(isSelectingLine)
				{
					MoveNext();
				}
				else if(isLeaving)
				{
					MoveBack();
				}
 			}
		}
	}

	public void ClearAttributes()
	{
		TargetPosition = Vector3.zero;
		isSelectingLine = false;
		isLeaving = false;
		CurrentPlace = -1;
		CurrentLine = -1;
		IsMoving = false;
		CurrentLineManager = null;
	}

	public void SetIsMoving(bool newIsMoving)
	{
		IsMoving = newIsMoving;
	}

	void MoveNext()
	{
		if(CurrentWaypoint >= NumWaypoints)
		{
			if(ShopManager.RequestLine(this, out CurrentPlace, out CurrentLine, out CurrentLineManager))
			{
				TargetPosition = CurrentLineManager.GetPlacePosition(CurrentPlace);
				MoveTo(TargetPosition);
			}
			else
			{
				isLeaving = true;
				CurrentWaypoint--;
				MoveBack();
			}

			isSelectingLine = false;
		}
		else
		{
			MoveTo(PreviousWaypoints[CurrentWaypoint]);
			CurrentWaypoint++;
		}
	}

	void MoveBack()
	{
		CurrentWaypoint--;

		if(CurrentWaypoint < 0)
		{
			Destroy(gameObject);
			ShopManager.ClientLeft();
		}
		else
		{
			MoveTo(PreviousWaypoints[CurrentWaypoint]);
		}

	}

	public void MoveTo(Vector3 position)
	{
		TargetPosition = position;
		position = position - transform.position;
		// float angle = (Mathf.Atan2(position.y, position.x) * Mathf.Rad2Deg);
		// if(angle >= 180)
		// {
		// 	angle = -angle;

		// }
        // transform.rotation = Quaternion.AngleAxis(angle, Vector3.forward);

		IsMoving = true;
	}

	public void SetPlace(int newPlace)
	{
		CurrentPlace = newPlace;
	}

	public void SetCurrentLineManager(LineManager lineManager)
	{
		CurrentLineManager = lineManager;
	}

	public void SetIsLeaving(bool newIsLeaving)
	{
		isLeaving = newIsLeaving;
	}

	public void SetBeingHold(bool newIsBeingHold)
	{
		IsBeingHold = newIsBeingHold;
	}

	public void ResetRotation()
	{
		transform.eulerAngles = new Vector3(0, 180, 0);
	}

	public int GetPlace()
	{
		return CurrentPlace;
	}

	public int GetLine()
	{
		return CurrentLine;
	}

	public void SetLine(int newLine)
	{
		 CurrentLine = newLine;
	}

	public void SetServiceAmount(float newServiceAmount)
	{
		ServiceAmount = newServiceAmount;
	}

	public void ReceiveProduct(Product product)
	{
		if(product)
		{
			GameObject instantiated = Instantiate (product.gameObject, holdingPivot.transform.position, Quaternion.identity);
			ServiceAmount = product.GetWorkload();
      		instantiated.transform.parent = gameObject.transform;

			hasProduct = true;
			productBeingHold = product;
		}
	}

	public bool CanDoService()
	{
		return !IsMoving && !isSelectingLine && !isLeaving;
	}

	public void DoService(float Amount)
	{
		ServiceAmount -= Amount;

		if(ServiceAmount < 0)
		{
			CompleteService();
		}
	}

	void CompleteService()
	{
		CurrentLineManager.DeocuppyLinePlace();

		SoundManager.instance.PlaySingle(cashingClip, 0.9f);

		if(productBeingHold)
		{
			gameManager.AddMoney(productBeingHold.GetMoney());
		}
		else
		{
			gameManager.AddMoney(baseValue);
		}

		Destroy(gameObject);
	}

	IEnumerator CheckServiceAmount()
	{
		while(true)
		{
			if(CurrentLine == -1 || CurrentPlace == -1 || CurrentLineManager == null)
			{
				if(!isLeaving && !isSelectingLine && !IsBeingHold) 
				{
					LeaveStore();
				}
			}

			yield return new WaitForSeconds(checkingDelay);
		}
	}

	public void LeaveStore()
	{
		SoundManager.instance.PlaySingle(angryClip, 1.3f);

		isLeaving = true;
		IsMoving = true;
		CurrentWaypoint = 3;

		MoveBack();
	}
}
