﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ShopManager : MonoBehaviour {

	public GameObject ShopEnteringObject;
	private Vector3 ShopEnteringPosition;
	public Quaternion ShopEnteringRotation;
	public float SpawnPeriod;
	public float SpawnPeriodDecrement;
	public GameObject CustomerPrefab;
	public GameObject[] Lines;
	private LineManager[] LinesManager;
	private int NumLines;
	public GameObject[] PreviousWaypointsObj;
	private int place;

	private GameManager gameManager;
	public ProductHandler productHandler;

	// Use this for initialization
	void Start () {

		gameManager = GameObject.Find("GameManager").GetComponent<GameManager>();

		NumLines = Lines.Length;

		ShopEnteringRotation = CustomerPrefab.transform.rotation;

		ShopEnteringPosition = ShopEnteringObject.transform.position;

		LinesManager = new LineManager[NumLines];

		for(int i = 0; i < NumLines; i++)
		{
			LinesManager[i] = Lines[i].GetComponent<LineManager>();
		}

		StartCoroutine(StartSpawning());

		ToggleHolders();
		
	}

	public void FixedUpdate()
	{
		SpawnPeriod -= SpawnPeriodDecrement;
	}

	public GameObject[] GetPreviousWaypointsObj()
	{
		return PreviousWaypointsObj;
	}
	
	public bool RequestLine(Person PersonRef, out int place, out int line, out LineManager lineManager)
	{
		bool IsAllFull = true;

		int i = 0;

		while(IsAllFull && i < NumLines)
		{
			if(LinesManager[i].isLineFull() == false)
			{
				IsAllFull = false;
			} 
			else
			{
				i++;
			}
		}

		if(!IsAllFull)
		{
			while(true)
			{
				int lineIndex = Random.Range(0, NumLines);
				if(LinesManager[lineIndex].isLineFull() == false)
				{
					place = LinesManager[lineIndex].OcuppyLinePlace(PersonRef);
					line = lineIndex;
					lineManager = LinesManager[lineIndex];
					return true;
				}
			}
		}

		place = -1;
		line = -1;
		lineManager = null;

		return false;
	}

	// public void GetLaneAttributes(int lineIndex, out int place, out LineManager lineManager)
	// {
	// 	place = LinesManager[lineIndex].OcuppyLinePlace(PersonRef);
	// 	lineManager = LinesManager[lineIndex];
	// }

	IEnumerator StartSpawning()
	{
		yield return new WaitForSeconds(1f);
		while(true)
		{
			Instantiate(CustomerPrefab, ShopEnteringPosition, ShopEnteringRotation);

			yield return new WaitForSeconds(SpawnPeriod);
		}
	}

	public void RemoveFromLine(Person person)
	{
		if(person.GetLine() != -1 || person.GetPlace() != -1){
			LinesManager[person.GetLine()].RemovePersonAt(person.GetPlace());
		}	
	}

	public void ToggleHolders()
	{
		foreach (LineManager item in LinesManager)
		{
			item.ToggleHolder();
		}
	}

	public void ClientLeft()
	{
		gameManager.ReportUnhappyCustomer();
	}

	public Product ReturnRandomProduct()
	{
		return productHandler.GetRandomProduct();
	}
}
